@@ -1,5 +1,5 @@
 /*
-** Copyright (C) 1999-2002 Erik de Castro Lopo <erikd@zip.com.au>
+** Copyright (C) 1999-2004 Erik de Castro Lopo <erikd@mega-nerd.com>
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU Lesser General Public License as published by
@@ -16,29 +16,22 @@
 ** Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
 
-#include	<stdio.h>
-#include	<unistd.h>
-#include	<string.h>
-#include	<time.h>
-#include	<ctype.h>
+#include "config.h"
 
-#include	"sndfile.h"
-#include	"config.h"
-#include	"sfendian.h"
-#include	"common.h"
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+#include <time.h>
+#include <ctype.h>
+
+#include "sndfile.h"
+#include "sfendian.h"
+#include "common.h"
 
 /*------------------------------------------------------------------------------
  * Macros to handle big/little endian issues.
  */
 
-#if (CPU_IS_LITTLE_ENDIAN == 1)
-	#define	MAKE_MARKER(a,b,c,d)		((a)|((b)<<8)|((c)<<16)|((d)<<24))
-#elif (CPU_IS_BIG_ENDIAN == 1)
-	#define	MAKE_MARKER(a,b,c,d)		(((a)<<24)|((b)<<16)|((c)<<8)|(d))
-#else
-	#error "Cannot determine endian-ness of processor."
-#endif
-
 #define FORM_MARKER	(MAKE_MARKER ('F', 'O', 'R', 'M'))
 #define AIFF_MARKER	(MAKE_MARKER ('A', 'I', 'F', 'F'))
 #define AIFC_MARKER	(MAKE_MARKER ('A', 'I', 'F', 'C'))
@@ -56,8 +49,8 @@
 #define FVER_MARKER	(MAKE_MARKER ('F', 'V', 'E', 'R'))
 #define SFX_MARKER	(MAKE_MARKER ('S', 'F', 'X', '!'))
 
-
 #define PEAK_MARKER	(MAKE_MARKER ('P', 'E', 'A', 'K'))
+#define basc_MARKER	(MAKE_MARKER ('b', 'a', 's', 'c'))
 
 /* Supported AIFC encodings.*/
 #define NONE_MARKER	(MAKE_MARKER ('N', 'O', 'N', 'E'))
@@ -79,20 +72,28 @@
 
 #define DWVW_MARKER	(MAKE_MARKER ('D', 'W', 'V', 'W'))
 #define GSM_MARKER	(MAKE_MARKER ('G', 'S', 'M', ' '))
+#define ima4_MARKER	(MAKE_MARKER ('i', 'm', 'a', '4'))
 
 /* Unsupported AIFC encodings.*/
-#define ima4_MARKER	(MAKE_MARKER ('i', 'm', 'a', '4'))
 
 #define MAC3_MARKER	(MAKE_MARKER ('M', 'A', 'C', '3'))
 #define MAC6_MARKER	(MAKE_MARKER ('M', 'A', 'C', '6'))
 #define ADP4_MARKER	(MAKE_MARKER ('A', 'D', 'P', '4'))
 
 /* Predfined chunk sizes. */
-#define SIZEOF_AIFF_COMM	18
+#define SIZEOF_AIFF_COMM		18
 #define SIZEOF_AIFC_COMM_MIN	22
-#define SIZEOF_AIFC_COMM	24
-#define SIZEOF_SSND_CHUNK	8
-#define SIZEOF_INST_CHUNK	20
+#define SIZEOF_AIFC_COMM		24
+#define SIZEOF_SSND_CHUNK		8
+#define SIZEOF_INST_CHUNK		20
+
+/* Is it constant? */
+#define SIZEOF_basc_CHUNK				0x54
+#define SIZEOF_basc_CHUNK_PADDING		66
+
+/* AIFC/IMA4 defines. */
+#define AIFC_IMA4_BLOCK_LEN				34
+#define	AIFC_IMA4_SAMPLES_PER_BLOCK		64
 
 /*------------------------------------------------------------------------------
  * Typedefs for file chunks.
@@ -106,8 +107,7 @@ enum
 } ;
 
 typedef struct
-{	unsigned int	type ;
-	unsigned int	size ;
+{	unsigned int	size ;
 	short			numChannels ;
 	unsigned int	numSampleFrames ;
 	short			sampleSize ;
@@ -135,15 +135,40 @@ typedef struct
 	char		lowVelocity ;	/* 1 to 127 */
 	char		highVelocity ;	/* 1 to 127 */
 	short		gain ;			/* in dB, 0 is normal */
-	INST_LOOP   sustain_loop ;
+	INST_LOOP	sustain_loop ;
 	INST_LOOP	release_loop ;
 } INST_CHUNK ;
 
+
+enum
+{	basc_SCALE_MINOR = 1,
+	basc_SCALE_MAJOR,
+	basc_SCALE_NEITHER,
+	basc_SCALE_BOTH,
+} ;
+
+enum
+{	basc_TYPE_LOOP = 0,
+	basc_TYPE_ONE_SHOT,
+} ;
+
+
+typedef struct
+{	unsigned int	version ;
+	unsigned int	numBeats ;
+	unsigned short	rootNote ;
+	unsigned short	scaleType ;
+	unsigned short	sigNumerator ;
+	unsigned short	sigDenominator ;
+	unsigned short	loopType ;
+	char			zero_bytes [SIZEOF_basc_CHUNK_PADDING] ;
+} basc_CHUNK ;
+
 /*------------------------------------------------------------------------------
  * Private static functions.
  */
 
-static int	aiff_close	(SF_PRIVATE  *psf) ;
+static int	aiff_close	(SF_PRIVATE *psf) ;
 
 static int	tenbytefloat2int (unsigned char *bytes) ;
 static void	uint2tenbytefloat (unsigned int num, unsigned char *bytes) ;
@@ -154,11 +179,13 @@ static int	aiff_read_header (SF_PRIVATE *psf, COMM_CHUNK *comm_fmt) ;
 
 static int	aiff_write_header (SF_PRIVATE *psf, int calc_length) ;
 static int	aiff_write_tailer (SF_PRIVATE *psf) ;
+static void aiff_write_strings (SF_PRIVATE *psf, int location) ;
 
 static int	aiff_command (SF_PRIVATE *psf, int command, void *data, int datasize) ;
 
-static char *get_loop_mode_str (short mode) ;
+static const char *get_loop_mode_str (short mode) ;
 
+static int aiff_read_basc_chunk (SF_PRIVATE * psf) ;
 
 /*------------------------------------------------------------------------------
 ** Public function.
@@ -179,14 +206,29 @@ aiff_open	(SF_PRIVATE *psf)
 		} ;
 
 	if (psf->mode == SFM_WRITE || psf->mode == SFM_RDWR)
-	{	if ((psf->sf.format & SF_FORMAT_TYPEMASK) != SF_FORMAT_AIFF)
+	{	if (psf->is_pipe)
+			return SFE_NO_PIPE_WRITE ;
+
+		if ((psf->sf.format & SF_FORMAT_TYPEMASK) != SF_FORMAT_AIFF)
 			return	SFE_BAD_OPEN_FORMAT ;
 
 		if (psf->mode == SFM_WRITE && (subformat == SF_FORMAT_FLOAT || subformat == SF_FORMAT_DOUBLE))
-		{	psf->has_peak = SF_TRUE ;
+		{	psf->pchunk = calloc (1, sizeof (PEAK_CHUNK) * psf->sf.channels * sizeof (PEAK_POS)) ;
+			if (psf->pchunk == NULL)
+				return SFE_MALLOC_FAILED ;
+			psf->has_peak = SF_TRUE ;
 			psf->peak_loc = SF_PEAK_START ;
 			} ;
 
+		if (psf->mode != SFM_RDWR || psf->filelength < 40)
+		{	psf->filelength = 0 ;
+			psf->datalength = 0 ;
+			psf->dataoffset = 0 ;
+			psf->sf.frames = 0 ;
+			} ;
+
+		psf->str_flags = SF_STR_ALLOW_START | SF_STR_ALLOW_END ;
+
 		if ((error = aiff_write_header (psf, SF_FALSE)))
 			return error ;
 
@@ -196,7 +238,7 @@ aiff_open	(SF_PRIVATE *psf)
 	psf->close = aiff_close ;
 	psf->command = aiff_command ;
 
-	psf->blockwidth  = psf->sf.channels * psf->bytewidth ;
+	psf->blockwidth = psf->sf.channels * psf->bytewidth ;
 
 	switch (psf->sf.format & SF_FORMAT_SUBMASK)
 	{	case SF_FORMAT_PCM_U8 :
@@ -213,14 +255,6 @@ aiff_open	(SF_PRIVATE *psf)
 				error = pcm_init (psf) ;
 				break ;
 
-		case SF_FORMAT_FLOAT :
-				error = float32_init (psf) ;
-				break ;
-
-		case SF_FORMAT_DOUBLE :
-				error = double64_init (psf) ;
-				break ;
-
 		case SF_FORMAT_ULAW :
 				error = ulaw_init (psf) ;
 				break ;
@@ -229,6 +263,15 @@ aiff_open	(SF_PRIVATE *psf)
 				error = alaw_init (psf) ;
 				break ;
 
+		/* Lite remove start */
+		case SF_FORMAT_FLOAT :
+				error = float32_init (psf) ;
+				break ;
+
+		case SF_FORMAT_DOUBLE :
+				error = double64_init (psf) ;
+				break ;
+
 		case SF_FORMAT_DWVW_12 :
 				error = dwvw_init (psf, 12) ;
 				break ;
@@ -251,20 +294,23 @@ aiff_open	(SF_PRIVATE *psf)
 					psf->sf.frames = comm_fmt.numSampleFrames ;
 					break ;
 					} ;
-				psf_log_printf (psf, "AIFC : Bad bitwidth %d\n", comm_fmt.sampleSize) ;
+				psf_log_printf (psf, "AIFC/DWVW : Bad bitwidth %d\n", comm_fmt.sampleSize) ;
 				error = SFE_DWVW_BAD_BITWIDTH ;
 				break ;
 
+		case SF_FORMAT_IMA_ADPCM :
+				/*
+				**	IMA ADPCM encoded AIFF files always have a block length
+				**	of 34 which decodes to 64 samples.
+				*/
+				error = aiff_ima_init (psf, AIFC_IMA4_BLOCK_LEN, AIFC_IMA4_SAMPLES_PER_BLOCK) ;
+				break ;
+		/* Lite remove end */
+
 		case SF_FORMAT_GSM610 :
 				error = gsm610_init (psf) ;
 				break ;
 
-		/*
-		case SF_FORMAT_IMA_ADPCM :
-				error = aiff_ima_writer_init (psf, 34) ;
-				break ;
-		*/
-
 		default : return SFE_UNIMPLEMENTED ;
 		} ;
 
@@ -281,7 +327,7 @@ aiff_open	(SF_PRIVATE *psf)
 static int
 aiff_read_header (SF_PRIVATE *psf, COMM_CHUNK *comm_fmt)
 {	SSND_CHUNK	ssnd_fmt ;
-	int			marker, dword, bytesread ;
+	int			marker, dword, bytesread, k ;
 	int			FORMsize, SSNDsize ;
 	int			filetype, found_chunk = 0, done = 0, error = 0 ;
 	char		*cptr, byte ;
@@ -300,6 +346,10 @@ aiff_read_header (SF_PRIVATE *psf, COMM_CHUNK *comm_fmt)
 	*/
 	while (! done)
 	{	psf_binheader_readf (psf, "m", &marker) ;
+
+		if (psf->mode == SFM_RDWR && (found_chunk & HAVE_SSND))
+			return SFE_AIFF_RW_SSND_NOT_LAST ;
+
 		switch (marker)
 		{	case FORM_MARKER :
 					if (found_chunk)
@@ -307,8 +357,13 @@ aiff_read_header (SF_PRIVATE *psf, COMM_CHUNK *comm_fmt)
 
 					psf_binheader_readf (psf, "E4", &FORMsize) ;
 
-					if (FORMsize != psf->filelength - 2 * SIGNED_SIZEOF (dword))
-					{	dword = psf->filelength - 2 * sizeof (dword);
+					if (psf->fileoffset > 0 && psf->filelength > FORMsize + 8)
+					{	/* Set file length. */
+						psf->filelength = FORMsize + 8 ;
+						psf_log_printf (psf, "FORM : %u\n", FORMsize) ;
+						}
+					else if (FORMsize != psf->filelength - 2 * SIGNED_SIZEOF (dword))
+					{	dword = psf->filelength - 2 * sizeof (dword) ;
 						psf_log_printf (psf, "FORM : %u (should be %u)\n", FORMsize, dword) ;
 						FORMsize = dword ;
 						}
@@ -348,38 +403,37 @@ aiff_read_header (SF_PRIVATE *psf, COMM_CHUNK *comm_fmt)
 					psf_binheader_readf (psf, "E4", &dword) ;
 
 					psf_log_printf (psf, "%M : %d\n", marker, dword) ;
-					if (dword > SIGNED_SIZEOF (psf->peak))
+					if (dword != SIGNED_SIZEOF (PEAK_CHUNK) + psf->sf.channels * SIGNED_SIZEOF (PEAK_POS))
 					{	psf_binheader_readf (psf, "j", dword) ;
 						psf_log_printf (psf, "*** File PEAK chunk bigger than sizeof (PEAK_CHUNK).\n") ;
 						return SFE_WAV_BAD_PEAK ;
 						} ;
-					if (dword != SIGNED_SIZEOF (psf->peak) - SIGNED_SIZEOF (psf->peak.peak) + psf->sf.channels * SIGNED_SIZEOF (PEAK_POS))
-					{	psf_binheader_readf (psf, "j", dword) ;
-						psf_log_printf (psf, "*** File PEAK chunk size doesn't fit with number of channels.\n") ;
-						return SFE_WAV_BAD_PEAK ;
-						} ;
+
+					psf->pchunk = calloc (1, sizeof (PEAK_CHUNK) * psf->sf.channels * sizeof (PEAK_POS)) ;
+					if (psf->pchunk == NULL)
+						return SFE_MALLOC_FAILED ;
 
 					/* read in rest of PEAK chunk. */
-					psf_binheader_readf (psf, "E44", &(psf->peak.version), &(psf->peak.timestamp)) ;
+					psf_binheader_readf (psf, "E44", &(psf->pchunk->version), &(psf->pchunk->timestamp)) ;
 
-					if (psf->peak.version != 1)
-						psf_log_printf (psf, "  version    : %d *** (should be version 1)\n", psf->peak.version) ;
+					if (psf->pchunk->version != 1)
+						psf_log_printf (psf, "  version    : %d *** (should be version 1)\n", psf->pchunk->version) ;
 					else
-						psf_log_printf (psf, "  version    : %d\n", psf->peak.version) ;
+						psf_log_printf (psf, "  version    : %d\n", psf->pchunk->version) ;
 
-					psf_log_printf (psf, "  time stamp : %d\n", psf->peak.timestamp) ;
+					psf_log_printf (psf, "  time stamp : %d\n", psf->pchunk->timestamp) ;
 					psf_log_printf (psf, "    Ch   Position       Value\n") ;
 
-					cptr = (char *) psf->buffer ;
+					cptr = psf->u.scbuf ;
 					for (dword = 0 ; dword < psf->sf.channels ; dword++)
-					{	psf_binheader_readf (psf, "Ef4", &(psf->peak.peak[dword].value),
-														&(psf->peak.peak[dword].position)) ;
+					{	psf_binheader_readf (psf, "Ef4", &(psf->pchunk->peaks [dword].value),
+														&(psf->pchunk->peaks [dword].position)) ;
 
-						LSF_SNPRINTF (cptr, sizeof (psf->buffer), "    %2d   %-12d   %g\n",
-								dword, psf->peak.peak[dword].position, psf->peak.peak[dword].value) ;
-						cptr [sizeof (psf->buffer) - 1] = 0 ;
+						LSF_SNPRINTF (cptr, sizeof (psf->u.scbuf), "    %2d   %-12d   %g\n",
+								dword, psf->pchunk->peaks [dword].position, psf->pchunk->peaks [dword].value) ;
+						cptr [sizeof (psf->u.scbuf) - 1] = 0 ;
 						psf_log_printf (psf, cptr) ;
-						};
+						} ;
 
 					psf->has_peak = SF_TRUE ; /* Found PEAK chunk. */
 					break ;
@@ -395,12 +449,14 @@ aiff_read_header (SF_PRIVATE *psf, COMM_CHUNK *comm_fmt)
 						psf->datalength = psf->filelength - psf->dataoffset ;
 						}
 					else
-					{	psf_log_printf (psf, " SSND : %u\n", SSNDsize) ;
-						psf->dataend = psf->datalength + psf->dataoffset ;
-						} ;
+						psf_log_printf (psf, " SSND : %u\n", SSNDsize) ;
 
-					psf_log_printf (psf, "  Offset     : %d\n", ssnd_fmt.offset) ;
-					psf_log_printf (psf, "  Block Size : %d\n", ssnd_fmt.blocksize) ;
+					/* Only set dataend if there really is data at the end. */
+					if (psf->datalength + psf->dataoffset < psf->filelength)
+						psf->dataend = psf->datalength + psf->dataoffset ;
+
+					psf_log_printf (psf, "  Offset     : %u\n", ssnd_fmt.offset) ;
+					psf_log_printf (psf, "  Block Size : %u\n", ssnd_fmt.blocksize) ;
 
 					found_chunk |= HAVE_SSND ;
 
@@ -408,25 +464,126 @@ aiff_read_header (SF_PRIVATE *psf, COMM_CHUNK *comm_fmt)
 						break ;
 
 					/* 	Seek to end of SSND chunk. */
-					psf_fseek (psf, psf->dataoffset + psf->datalength, SEEK_SET) ;
+					psf_fseek (psf, psf->dataoffset + psf->datalength + (SSNDsize & 1), SEEK_SET) ;
 					break ;
 
 			case c_MARKER :
-			case ANNO_MARKER :
+					psf_binheader_readf (psf, "E4", &dword) ;
+					if (dword == 0)
+						break ;
+					if (dword > SIGNED_SIZEOF (psf->u.scbuf) - 1)
+					{	psf_log_printf (psf, " %M : %d (too big)\n", marker, dword) ;
+						return SFE_INTERNAL ;
+						} ;
+
+					cptr = psf->u.scbuf ;
+					psf_binheader_readf (psf, "b", cptr, dword) ;
+					cptr [dword] = 0 ;
+					psf_log_printf (psf, " %M : %s\n", marker, cptr) ;
+					psf_store_string (psf, SF_STR_COPYRIGHT, cptr) ;
+					break ;
+
 			case AUTH_MARKER :
-			case COMT_MARKER :
-			case NAME_MARKER :
-					psf_binheader_readf (psf, "E4", &dword);
+					psf_binheader_readf (psf, "E4", &dword) ;
 					dword += (dword & 1) ;
 					if (dword == 0)
 						break ;
-					if (dword > SIGNED_SIZEOF (psf->buffer))
-					{	psf_log_printf (psf, "  %M : %d (too big)\n", marker, dword) ;
+					if (dword > SIGNED_SIZEOF (psf->u.scbuf))
+					{	psf_log_printf (psf, " %M : %d (too big)\n", marker, dword) ;
 						return SFE_INTERNAL ;
 						} ;
-					psf_binheader_readf (psf, "b", psf->buffer, dword) ;
-					psf->buffer [dword - 1] = 0 ;
-					psf_log_printf (psf, "    %M : %s\n", marker, psf->buffer) ;
+
+					cptr = psf->u.scbuf ;
+					psf_binheader_readf (psf, "b", cptr, dword) ;
+					cptr [dword - 1] = 0 ;
+					psf_log_printf (psf, " %M : %s\n", marker, cptr) ;
+					psf_store_string (psf, SF_STR_ARTIST, cptr) ;
+					break ;
+
+			case COMT_MARKER :
+				{	unsigned short count, id, len ;
+					unsigned int timestamp ;
+
+					psf_binheader_readf (psf, "E42", &dword, &count) ;
+					psf_log_printf (psf, " %M : %d\n  count  : %d\n", marker, dword, count) ;
+					dword += (dword & 1) ;
+					if (dword == 0)
+						break ;
+
+					for (k = 0 ; k < count ; k++)
+					{	psf_binheader_readf (psf, "E422", &timestamp, &id, &len) ;
+
+						psf_log_printf (psf, "   time   : 0x%x\n   marker : %x\n   length : %d\n", timestamp, id, len) ;
+
+						if (len + 1 > SIGNED_SIZEOF (psf->u.scbuf))
+						{	psf_log_printf (psf, "\nError : string length (%d) too big.\n", len) ;
+							return SFE_INTERNAL ;
+							} ;
+
+						cptr = psf->u.scbuf ;
+						psf_binheader_readf (psf, "b", cptr, len) ;
+						cptr [len] = 0 ;
+						psf_log_printf (psf, "   string : %s\n", cptr) ;
+						} ;
+					} ;
+					break ;
+
+			case APPL_MARKER :
+					psf_binheader_readf (psf, "E4", &dword) ;
+					dword += (dword & 1) ;
+					if (dword == 0)
+						break ;
+					if (dword >= SIGNED_SIZEOF (psf->u.scbuf))
+					{	psf_log_printf (psf, " %M : %d (too big, skipping)\n", marker, dword) ;
+						psf_binheader_readf (psf, "j", dword) ;
+						break ;
+						} ;
+
+					cptr = psf->u.scbuf ;
+					psf_binheader_readf (psf, "b", cptr, dword) ;
+					cptr [dword - 1] = 0 ;
+
+					for (k = 0 ; k < dword ; k++)
+						if (! isprint (cptr [k]))
+						{	cptr [k] = 0 ;
+							break ;
+							} ;
+
+					psf_log_printf (psf, " %M : %s\n", marker, cptr) ;
+					psf_store_string (psf, SF_STR_SOFTWARE, cptr) ;
+					break ;
+
+			case NAME_MARKER :
+					psf_binheader_readf (psf, "E4", &dword) ;
+					dword += (dword & 1) ;
+					if (dword == 0)
+						break ;
+					if (dword > SIGNED_SIZEOF (psf->u.scbuf))
+					{	psf_log_printf (psf, " %M : %d (too big)\n", marker, dword) ;
+						return SFE_INTERNAL ;
+						} ;
+
+					cptr = psf->u.scbuf ;
+					psf_binheader_readf (psf, "b", cptr, dword) ;
+					cptr [dword - 1] = 0 ;
+					psf_log_printf (psf, " %M : %s\n", marker, cptr) ;
+					psf_store_string (psf, SF_STR_TITLE, cptr) ;
+					break ;
+
+			case ANNO_MARKER :
+					psf_binheader_readf (psf, "E4", &dword) ;
+					if (dword == 0)
+						break ;
+					if (dword > SIGNED_SIZEOF (psf->u.scbuf) - 1)
+					{	psf_log_printf (psf, " %M : %d (too big)\n", marker, dword) ;
+						return SFE_INTERNAL ;
+						} ;
+
+					cptr = psf->u.scbuf ;
+					psf_binheader_readf (psf, "b", cptr, dword) ;
+					cptr [dword] = 0 ;
+					psf_log_printf (psf, " %M : %s\n", marker, cptr) ;
+					psf_store_string (psf, SF_STR_COMMENT, cptr) ;
 					break ;
 
 			case INST_MARKER :
@@ -450,8 +607,8 @@ aiff_read_header (SF_PRIVATE *psf, COMM_CHUNK *comm_fmt)
 						psf_log_printf (psf, 	"  Gain (dB) : %d\n", gain) ;
 
 						} ;
-					{	short	mode ;  /* 0 - no loop, 1 - forward looping, 2 - backward looping */
-						char	*loop_mode ;
+					{	short	mode ; /* 0 - no loop, 1 - forward looping, 2 - backward looping */
+						const char	*loop_mode ;
 						unsigned short begin, end ;
 
 						psf_binheader_readf (psf, "E222", &mode, &begin, &end) ;
@@ -465,11 +622,26 @@ aiff_read_header (SF_PRIVATE *psf, COMM_CHUNK *comm_fmt)
 						} ;
 					break ;
 
+			case basc_MARKER :
+					psf_binheader_readf (psf, "E4", &dword) ;
+
+					if (dword != SIZEOF_basc_CHUNK)
+					{	psf_log_printf (psf, " %M : %d (should be %d)\n", marker, dword, SIZEOF_basc_CHUNK) ;
+						psf_binheader_readf (psf, "j", dword) ;
+						break ;
+						} ;
+
+					psf_log_printf (psf, " basc : %u\n", dword) ;
+
+					if ((error = aiff_read_basc_chunk (psf)))
+						return error ;
+					break ;
+
 			case MARK_MARKER :
 					psf_binheader_readf (psf, "E4", &dword) ;
 					psf_log_printf (psf, " %M : %d\n", marker, dword) ;
-					{	unsigned short  mark_count, mark_id ;
-						unsigned char   pstr_len ;
+					{	unsigned short	mark_count, mark_id ;
+						unsigned char	pstr_len ;
 						unsigned int	position ;
 
 						bytesread = psf_binheader_readf (psf, "E2", &mark_count) ;
@@ -479,10 +651,10 @@ aiff_read_header (SF_PRIVATE *psf, COMM_CHUNK *comm_fmt)
 						{	bytesread += psf_binheader_readf (psf, "E241", &mark_id, &position, &pstr_len) ;
 							psf_log_printf (psf, "   Mark ID  : %u\n   Position : %u\n", mark_id, position) ;
 
-							pstr_len += (pstr_len & 1) + 1 ;  /* fudgy, fudgy, hack, hack */
+							pstr_len += (pstr_len & 1) + 1 ; /* fudgy, fudgy, hack, hack */
 
-							bytesread += psf_binheader_readf (psf, "b", psf->buffer, pstr_len) ;
-							psf_log_printf (psf, "   Name     : %s\n", psf->buffer) ;
+							bytesread += psf_binheader_readf (psf, "b", psf->u.scbuf, pstr_len) ;
+							psf_log_printf (psf, "   Name     : %s\n", psf->u.scbuf) ;
 
 							mark_count -- ;
 							} ;
@@ -490,7 +662,6 @@ aiff_read_header (SF_PRIVATE *psf, COMM_CHUNK *comm_fmt)
 					psf_binheader_readf (psf, "j", dword - bytesread) ;
 					break ;
 
-			case APPL_MARKER :
 			case FVER_MARKER :
 			case SFX_MARKER :
 					psf_binheader_readf (psf, "E4", &dword) ;
@@ -510,18 +681,18 @@ aiff_read_header (SF_PRIVATE *psf, COMM_CHUNK *comm_fmt)
 					if (isprint ((marker >> 24) & 0xFF) && isprint ((marker >> 16) & 0xFF)
 						&& isprint ((marker >> 8) & 0xFF) && isprint (marker & 0xFF))
 					{	psf_binheader_readf (psf, "E4", &dword) ;
-						psf_log_printf (psf, "%M : %d (unknown marker)\n", marker, dword) ;
+						psf_log_printf (psf, " %M : %d (unknown marker)\n", marker, dword) ;
 
 						psf_binheader_readf (psf, "j", dword) ;
 						break ;
 						} ;
 					if ((dword = psf_ftell (psf)) & 0x03)
-					{	psf_log_printf (psf, "  Unknown chunk marker at position %d. Resyncing.\n", dword - 4) ;
+					{	psf_log_printf (psf, "  Unknown chunk marker %X at position %d. Resyncing.\n", marker, dword - 4) ;
 
 						psf_binheader_readf (psf, "j", -3) ;
 						break ;
 						} ;
-					psf_log_printf (psf, "*** Unknown chunk marker : %X at position %d. Exiting parser.\n", marker, psf_ftell (psf)) ;
+					psf_log_printf (psf, "*** Unknown chunk marker %X at position %D. Exiting parser.\n", marker, psf_ftell (psf)) ;
 					done = 1 ;
 					break ;
 			} ;	/* switch (marker) */
@@ -529,10 +700,10 @@ aiff_read_header (SF_PRIVATE *psf, COMM_CHUNK *comm_fmt)
 		if ((! psf->sf.seekable) && (found_chunk & HAVE_SSND))
 			break ;
 
-		if (psf_ftell (psf) >= psf->filelength - (2 * sizeof (dword)))
+		if (psf_ftell (psf) >= psf->filelength - (2 * SIGNED_SIZEOF (dword)))
 			break ;
 
-		if (psf->logindex >= sizeof (psf->logbuffer) - 2)
+		if (psf->logindex >= SIGNED_SIZEOF (psf->logbuffer) - 2)
 			return SFE_LOG_OVERRUN ;
 		} ; /* while (1) */
 
@@ -552,31 +723,14 @@ aiff_read_header (SF_PRIVATE *psf, COMM_CHUNK *comm_fmt)
 } /* aiff_read_header */
 
 static int
-aiff_close	(SF_PRIVATE  *psf)
+aiff_close	(SF_PRIVATE *psf)
 {
 	if (psf->mode == SFM_WRITE || psf->mode == SFM_RDWR)
-	{	/*  Now we know for certain the length of the file we can re-write
-		**	correct values for the FORM, COMM and SSND chunks.
-		*/
+	{	aiff_write_tailer (psf) ;
 
-		psf_fseek (psf, 0, SEEK_END) ;
-		psf->dataend = psf_ftell (psf) ;
-		aiff_write_tailer (psf) ;
-
-		psf_fseek (psf, 0, SEEK_END) ;
-		psf->filelength = psf_ftell (psf) ;
-		psf_fseek (psf, 0, SEEK_SET) ;
-
-		psf->datalength = psf->filelength - psf->dataoffset - (psf->filelength - psf->dataend) ;
- 		psf->sf.frames = psf->datalength / (psf->bytewidth * psf->sf.channels) ;
-
-		aiff_write_header (psf, SF_FALSE) ;
+		aiff_write_header (psf, SF_TRUE) ;
 		} ;
 
-	if (psf->fdata)
-		free (psf->fdata) ;
-	psf->fdata = NULL ;
-
 	return 0 ;
 } /* aiff_close */
 
@@ -584,12 +738,15 @@ static int
 aiff_read_comm_chunk (SF_PRIVATE *psf, COMM_CHUNK *comm_fmt)
 {	int error = 0, bytesread, subformat ;
 
+	psf->u.scbuf [0] = 0 ;
+
 	bytesread = psf_binheader_readf (psf, "E4", &(comm_fmt->size)) ;
 
 	/* The COMM chunk has an int aligned to an odd word boundary. Some
 	** procesors are not able to deal with this (ie bus fault) so we have
 	** to take special care.
 	*/
+	comm_fmt->size += comm_fmt->size & 1 ;
 
 	bytesread +=
 	psf_binheader_readf (psf, "E242b", &(comm_fmt->numChannels), &(comm_fmt->numSampleFrames),
@@ -604,16 +761,16 @@ aiff_read_comm_chunk (SF_PRIVATE *psf, COMM_CHUNK *comm_fmt)
 
 		bytesread += psf_binheader_readf (psf, "Em1", &(comm_fmt->encoding), &encoding_len) ;
 
-		memset (psf->buffer, 0, comm_fmt->size) ;
+		memset (psf->u.scbuf, 0, comm_fmt->size) ;
 
-		bytesread += psf_binheader_readf (psf, "b", psf->buffer,
+		bytesread += psf_binheader_readf (psf, "b", psf->u.scbuf,
 							comm_fmt->size - SIZEOF_AIFC_COMM + 1) ;
-		psf->buffer [encoding_len] = 0 ;
+		psf->u.scbuf [encoding_len] = 0 ;
 		} ;
 
 	psf_log_printf (psf, " COMM : %d\n", comm_fmt->size) ;
 	psf_log_printf (psf, "  Sample Rate : %d\n", tenbytefloat2int (comm_fmt->sampleRate)) ;
-	psf_log_printf (psf, "  Frames      : %d\n", comm_fmt->numSampleFrames) ;
+	psf_log_printf (psf, "  Frames      : %u%s\n", comm_fmt->numSampleFrames, (comm_fmt->numSampleFrames == 0 && psf->filelength > 100) ? " (Should not be 0)" : "") ;
 	psf_log_printf (psf, "  Channels    : %d\n", comm_fmt->numChannels) ;
 
 	/*	Found some broken 'fl32' files with comm.samplesize == 16. Fix it here. */
@@ -689,7 +846,7 @@ aiff_read_comm_chunk (SF_PRIVATE *psf, COMM_CHUNK *comm_fmt)
 					default :
 						psf->sf.format |= SF_FORMAT_DWVW_N ;
 						break ;
-					}  ;
+					} ;
 				break ;
 
 		case GSM_MARKER :
@@ -698,22 +855,20 @@ aiff_read_comm_chunk (SF_PRIVATE *psf, COMM_CHUNK *comm_fmt)
 				break ;
 
 
-		/*
 		case ima4_MARKER :
 				psf->endian = SF_ENDIAN_BIG ;
 				psf->sf.format = (SF_FORMAT_AIFF | SF_FORMAT_IMA_ADPCM) ;
 				break ;
-		*/
 
 		default :
 			psf_log_printf (psf, "AIFC : Unimplemented format : %M\n", comm_fmt->encoding) ;
 			error = SFE_UNIMPLEMENTED ;
 		} ;
 
-	if (! psf->buffer [0])
+	if (! psf->u.scbuf [0])
 		psf_log_printf (psf, "  Encoding    : %M\n", comm_fmt->encoding) ;
 	else
-		psf_log_printf (psf, "  Encoding    : %M => %s\n", comm_fmt->encoding, (char*) psf->buffer) ;
+		psf_log_printf (psf, "  Encoding    : %M => %s\n", comm_fmt->encoding, psf->u.scbuf) ;
 
 	return error ;
 } /* aiff_read_comm_chunk */
@@ -722,22 +877,57 @@ static int
 aiff_write_header (SF_PRIVATE *psf, int calc_length)
 {	sf_count_t		current ;
 	unsigned char 	comm_sample_rate [10], comm_zero_bytes [2] = { 0, 0 } ;
-	unsigned int 	comm_type, comm_size, comm_encoding ;
+	unsigned int 	comm_type, comm_size, comm_encoding, comm_frames ;
 	int				k, endian ;
 	short			bit_width ;
 
 	current = psf_ftell (psf) ;
 
 	if (calc_length)
-	{	psf_fseek (psf, 0, SEEK_END) ;
-		psf->filelength = psf_ftell (psf) ;
-		psf_fseek (psf, 0, SEEK_SET) ;
+	{	psf->filelength = psf_get_filelen (psf) ;
 
 		psf->datalength = psf->filelength - psf->dataoffset ;
 		if (psf->dataend)
 			psf->datalength -= psf->filelength - psf->dataend ;
 
-		psf->sf.frames = psf->datalength / (psf->bytewidth * psf->sf.channels) ;
+		if (psf->bytewidth > 0)
+			psf->sf.frames = psf->datalength / (psf->bytewidth * psf->sf.channels) ;
+		} ;
+
+	if (psf->mode == SFM_RDWR && psf->dataoffset > 0)
+	{	/* Assuming here that the header has already been written and just
+		** needs to be corrected for new data length. That means that we
+		** only change the length fields of the FORM and SSND chunks;
+		** everything else can be skipped over.
+		*/
+
+		/* First write new FORM chunk. */
+		psf->headindex = 0 ;
+		psf_fseek (psf, 0, SEEK_SET) ;
+
+		psf_binheader_writef (psf, "Etm8", FORM_MARKER, psf->filelength - 8) ;
+		psf_fwrite (psf->header, psf->headindex, 1, psf) ;
+
+		/* Now write frame count field of COMM chunk header. */
+		psf->headindex = 0 ;
+		psf_fseek (psf, 22, SEEK_SET) ;
+
+		psf_binheader_writef (psf, "Et8", psf->sf.frames) ;
+		psf_fwrite (psf->header, psf->headindex, 1, psf) ;
+
+		/* Now write new SSND chunk header. */
+		psf->headindex = 0 ;
+		psf_fseek (psf, psf->dataoffset - 16, SEEK_SET) ;
+
+		psf_binheader_writef (psf, "Etm8", SSND_MARKER, psf->datalength + SIZEOF_SSND_CHUNK) ;
+		psf_fwrite (psf->header, psf->headindex, 1, psf) ;
+
+		if (current < psf->dataoffset)
+			psf_fseek (psf, psf->dataoffset, SEEK_SET) ;
+		else if (current > 0)
+			psf_fseek (psf, current, SEEK_SET) ;
+
+		return 0 ;
 		} ;
 
 	endian = psf->sf.format & SF_FORMAT_ENDMASK ;
@@ -746,6 +936,7 @@ aiff_write_header (SF_PRIVATE *psf, int calc_length)
 
 	/* Standard value here. */
 	bit_width = psf->bytewidth * 8 ;
+	comm_frames = (psf->sf.frames > 0xFFFFFFFF) ? 0xFFFFFFFF : psf->sf.frames ;
 
 	switch (psf->sf.format & SF_FORMAT_SUBMASK)
 	{	case SF_FORMAT_PCM_S8 :
@@ -780,14 +971,14 @@ aiff_write_header (SF_PRIVATE *psf, int calc_length)
 				psf->endian = SF_ENDIAN_BIG ;
 				comm_type = AIFC_MARKER ;
 				comm_size = SIZEOF_AIFC_COMM ;
-				comm_encoding = FL32_MARKER ; /* Use 'FL32' because its easier to read. */
+				comm_encoding = FL32_MARKER ;	/* Use 'FL32' because its easier to read. */
 				break ;
 
 		case SF_FORMAT_DOUBLE :					/* Big endian double precision floating point. */
 				psf->endian = SF_ENDIAN_BIG ;
 				comm_type = AIFC_MARKER ;
 				comm_size = SIZEOF_AIFC_COMM ;
-				comm_encoding = FL64_MARKER ; /* Use 'FL64' because its easier to read. */
+				comm_encoding = FL64_MARKER ;	/* Use 'FL64' because its easier to read. */
 				break ;
 
 		case SF_FORMAT_ULAW :
@@ -851,6 +1042,17 @@ aiff_write_header (SF_PRIVATE *psf, int calc_length)
 				bit_width = 16 ;
 				break ;
 
+		case SF_FORMAT_IMA_ADPCM :
+				psf->endian = SF_ENDIAN_BIG ;
+				comm_type = AIFC_MARKER ;
+				comm_size = SIZEOF_AIFC_COMM ;
+				comm_encoding = ima4_MARKER ;
+
+				/* Override standard value here.*/
+				bit_width = 16 ;
+				comm_frames = psf->sf.frames / AIFC_IMA4_SAMPLES_PER_BLOCK ;
+				break ;
+
 		default : return SFE_BAD_OPEN_FORMAT ;
 		} ;
 
@@ -864,21 +1066,24 @@ aiff_write_header (SF_PRIVATE *psf, int calc_length)
 	/* Write COMM chunk. */
 	psf_binheader_writef (psf, "Emm4", comm_type, COMM_MARKER, comm_size) ;
 
-	uint2tenbytefloat (psf->sf.samplerate, comm_sample_rate)  ;
+	uint2tenbytefloat (psf->sf.samplerate, comm_sample_rate) ;
 
-	psf_binheader_writef (psf, "Et282", psf->sf.channels, psf->sf.frames, bit_width) ;
+	psf_binheader_writef (psf, "Et242", psf->sf.channels, comm_frames, bit_width) ;
 	psf_binheader_writef (psf, "b", comm_sample_rate, sizeof (comm_sample_rate)) ;
 
 	/* AIFC chunks have some extra data. */
 	if (comm_type == AIFC_MARKER)
 		psf_binheader_writef (psf, "mb", comm_encoding, comm_zero_bytes, sizeof (comm_zero_bytes)) ;
 
+	if (psf->str_flags & SF_STR_LOCATE_START)
+		aiff_write_strings (psf, SF_STR_LOCATE_START) ;
+
 	if (psf->has_peak && psf->peak_loc == SF_PEAK_START)
 	{	psf_binheader_writef (psf, "Em4", PEAK_MARKER,
-			sizeof (psf->peak) - sizeof (psf->peak.peak) + psf->sf.channels * sizeof (PEAK_POS)) ;
+			sizeof (PEAK_CHUNK) + psf->sf.channels * sizeof (PEAK_POS)) ;
 		psf_binheader_writef (psf, "E44", 1, time (NULL)) ;
 		for (k = 0 ; k < psf->sf.channels ; k++)
-			psf_binheader_writef (psf, "Ef4", psf->peak.peak[k].value, psf->peak.peak[k].position) ; /* XXXXX */
+			psf_binheader_writef (psf, "Ef4", psf->pchunk->peaks [k].value, psf->pchunk->peaks [k].position) ; /* XXXXX */
 		} ;
 
 	/* Write SSND chunk. */
@@ -886,13 +1091,15 @@ aiff_write_header (SF_PRIVATE *psf, int calc_length)
 
 	/* Header construction complete so write it out. */
 	psf_fwrite (psf->header, psf->headindex, 1, psf) ;
-	
+
 	if (psf->error)
 		return psf->error ;
 
 	psf->dataoffset = psf->headindex ;
 
-	if (current > 0)
+	if (current < psf->dataoffset)
+		psf_fseek (psf, psf->dataoffset, SEEK_SET) ;
+	else if (current > 0)
 		psf_fseek (psf, current, SEEK_SET) ;
 
 	return psf->error ;
@@ -900,30 +1107,24 @@ aiff_write_header (SF_PRIVATE *psf, int calc_length)
 
 static int
 aiff_write_tailer (SF_PRIVATE *psf)
-{	static char inst_data [0x14] =
-	{	0x3C, 0x00, 0x00, 0x7F, 0x01, 0x7F, 0x00, 0x00, 0x00, 0x00,
-		0x00, 0x01, 0x00, 0x02, 0x00, 0x00, 0x00, 0x03, 0x00, 0x04
-		} ;
-	int		k ;
-
+{	int		k ;
 
 	/* Reset the current header length to zero. */
 	psf->header [0] = 0 ;
 	psf->headindex = 0 ;
-	psf_fseek (psf, 0, SEEK_END) ;
+
+	psf->dataend = psf_fseek (psf, 0, SEEK_END) ;
 
 	if (psf->has_peak && psf->peak_loc == SF_PEAK_END)
 	{	psf_binheader_writef (psf, "Em4", PEAK_MARKER,
-			sizeof (psf->peak) - sizeof (psf->peak.peak) + psf->sf.channels * sizeof (PEAK_POS)) ;
+			sizeof (PEAK_CHUNK) + psf->sf.channels * sizeof (PEAK_POS)) ;
 		psf_binheader_writef (psf, "E44", 1, time (NULL)) ;
 		for (k = 0 ; k < psf->sf.channels ; k++)
-			psf_binheader_writef (psf, "Ef4", psf->peak.peak[k].value, psf->peak.peak[k].position) ; /* XXXXX */
+			psf_binheader_writef (psf, "Ef4", psf->pchunk->peaks [k].value, psf->pchunk->peaks [k].position) ; /* XXXXX */
 		} ;
 
-	if (psf->add_test_tailer)
-	{	/* Add test INST chunk. */
-		psf_binheader_writef (psf, "Em4b", INST_MARKER, 0x14, inst_data, 0x14) ;
-		} ;
+	if (psf->str_flags & SF_STR_LOCATE_END)
+		aiff_write_strings (psf, SF_STR_LOCATE_END) ;
 
 	/* Write the tailer. */
 	if (psf->headindex)
@@ -932,29 +1133,69 @@ aiff_write_tailer (SF_PRIVATE *psf)
 	return 0 ;
 } /* aiff_write_tailer */
 
+static void
+aiff_write_strings (SF_PRIVATE *psf, int location)
+{	int	k ;
+
+	for (k = 0 ; k < SF_MAX_STRINGS ; k++)
+	{	if (psf->strings [k].type == 0)
+			break ;
+
+		if (psf->strings [k].flags != location)
+			continue ;
+
+		switch (psf->strings [k].type)
+		{	case SF_STR_SOFTWARE :
+				psf_binheader_writef (psf, "Ems", APPL_MARKER, psf->strings [k].str) ;
+				break ;
+
+			case SF_STR_TITLE :
+				psf_binheader_writef (psf, "Ems", NAME_MARKER, psf->strings [k].str) ;
+				break ;
+
+			case SF_STR_COPYRIGHT :
+				psf_binheader_writef (psf, "Ems", c_MARKER, psf->strings [k].str) ;
+				break ;
+
+			case SF_STR_ARTIST :
+				psf_binheader_writef (psf, "Ems", AUTH_MARKER, psf->strings [k].str) ;
+				break ;
+
+			case SF_STR_COMMENT :
+				psf_binheader_writef (psf, "Ems", ANNO_MARKER, psf->strings [k].str) ;
+				break ;
+
+			/*
+			case SF_STR_DATE :
+				psf_binheader_writef (psf, "Ems", ICRD_MARKER, psf->strings [k].str) ;
+				break ;
+			*/
+			} ;
+		} ;
+
+	return ;
+} /* aiff_write_strings */
+
 static int
 aiff_command (SF_PRIVATE *psf, int command, void *data, int datasize)
 {
 	/* Avoid compiler warnings. */
+	psf = psf ;
 	data = data ;
 	datasize = datasize ;
 
 	switch (command)
-	{	case SFC_TEST_ADD_TRAILING_DATA :
-			psf->add_test_tailer = SF_TRUE ;
-			break ;
-
-		default : break ;
+	{	default : break ;
 		} ;
 
 	return 0 ;
 } /* aiff_command */
 
-static char
-*get_loop_mode_str (short mode)
+static const char*
+get_loop_mode_str (short mode)
 {	switch (mode)
 	{	case 0 : return "none" ;
-		case 1 : return "forawrd" ;
+		case 1 : return "forward" ;
 		case 2 : return "backward" ;
 		} ;
 
@@ -972,23 +1213,23 @@ static char
 
 static int
 tenbytefloat2int (unsigned char *bytes)
-{       int val = 3 ;
+{	int val = 3 ;
 
-	if (bytes [0] & 0x80)   /* Negative number. */
+	if (bytes [0] & 0x80)	/* Negative number. */
 		return 0 ;
 
-	if (bytes [0] <= 0x3F)  /* Less than 1. */
+	if (bytes [0] <= 0x3F)	/* Less than 1. */
 		return 1 ;
 
-	if (bytes [0] > 0x40)   /* Way too big. */
-		return  0x4000000 ;
+	if (bytes [0] > 0x40)	/* Way too big. */
+		return 0x4000000 ;
 
 	if (bytes [0] == 0x40 && bytes [1] > 0x1C) /* Too big. */
 		return 800000000 ;
 
 	/* Ok, can handle it. */
 
-	val = (bytes [2] << 23) | (bytes [3] << 15) | (bytes [4] << 7)  | (bytes [5] >> 1) ;
+	val = (bytes [2] << 23) | (bytes [3] << 15) | (bytes [4] << 7) | (bytes [5] >> 1) ;
 
 	val >>= (29 - bytes [1]) ;
 
@@ -1027,8 +1268,78 @@ uint2tenbytefloat (unsigned int num, unsigned char *bytes)
 	bytes [1] = 29 - count ;
 	bytes [2] = (num >> 24) & 0xFF ;
 	bytes [3] = (num >> 16) & 0xFF ;
-	bytes [4] = (num >>  8) & 0xFF ;
+	bytes [4] = (num >> 8) & 0xFF ;
 	bytes [5] = num & 0xFF ;
 
 } /* uint2tenbytefloat */
 
+static int
+aiff_read_basc_chunk (SF_PRIVATE * psf)
+{	const char * type_str ;
+	basc_CHUNK bc ;
+
+	psf_binheader_readf (psf, "E442", &bc.version, &bc.numBeats, &bc.rootNote) ;
+	psf_binheader_readf (psf, "E222", &bc.scaleType, &bc.sigNumerator, &bc.sigDenominator) ;
+	psf_binheader_readf (psf, "E2b", &bc.loopType, &bc.zero_bytes, SIZEOF_basc_CHUNK_PADDING) ;
+
+	psf_log_printf (psf, "  Version ? : %u\n  Num Beats : %u\n  Root Note : 0x%x\n",
+						bc.version, bc.numBeats, bc.rootNote) ;
+
+	switch (bc.scaleType)
+	{	case basc_SCALE_MINOR :
+				type_str = "MINOR" ;
+				break ;
+		case basc_SCALE_MAJOR :
+				type_str = "MAJOR" ;
+				break ;
+		case basc_SCALE_NEITHER :
+				type_str = "NEITHER" ;
+				break ;
+		case basc_SCALE_BOTH :
+				type_str = "BOTH" ;
+				break ;
+		default :
+				type_str = "!!WRONG!!" ;
+				break ;
+		} ;
+
+	psf_log_printf (psf, 	"  ScaleType : 0x%x (%s)\n", bc.scaleType, type_str) ;
+	psf_log_printf (psf, 	"  Time Sig  : %d/%d\n", bc.sigNumerator, bc.sigDenominator) ;
+
+	switch (bc.loopType)
+	{	case basc_TYPE_ONE_SHOT :
+				type_str = "One Shot" ;
+				break ;
+		case basc_TYPE_LOOP :
+				type_str = "Loop" ;
+				break ;
+		default:
+				type_str = "!!WRONG!!" ;
+				break ;
+		} ;
+
+	psf_log_printf (psf, "  Loop Type : 0x%x (%s)\n", bc.loopType, type_str) ;
+
+	if ((psf->loop_info = calloc (1, sizeof (SF_LOOP_INFO))) == NULL)
+		return SFE_MALLOC_FAILED ;
+
+	psf->loop_info->time_sig_num	= bc.sigNumerator ;
+	psf->loop_info->time_sig_den	= bc.sigDenominator ;
+	psf->loop_info->loop_mode		= (bc.loopType == basc_TYPE_ONE_SHOT) ? SF_LOOP_NONE : SF_LOOP_FORWARD ;
+	psf->loop_info->num_beats		= bc.numBeats ;
+
+	/* Can always be recalculated from other known fields. */
+	psf->loop_info->bpm = (1.0 / psf->sf.frames) * psf->sf.samplerate
+							* ((bc.numBeats * 4.0) / bc.sigDenominator) * 60.0 ;
+	psf->loop_info->root_key = bc.rootNote ;
+
+	return 0 ;
+} /* aiff_read_basc_chunk */
+
+/*
+** Do not edit or modify anything in this comment block.
+** The arch-tag line is a file identity tag for the GNU Arch
+** revision control system.
+**
+** arch-tag: 7dec56ca-d6f2-48cf-863b-a72e7e17a5d9
+*/
