@@ -1,5 +1,5 @@
 /*
- * $Id: pa_jack.c,v 1.1.2.1 2004/04/22 04:39:41 mbrubeck Exp $
+ * $Id: pa_jack.c,v 1.1.2.4 2004/06/09 06:40:34 dmazzoni Exp $
  * PortAudio Portable Real-Time Audio Library
  * Latest Version at: http://www.portaudio.com
  * JACK Implementation by Joshua Haberman
@@ -266,25 +266,29 @@ static PaError BuildDeviceList( PaJackHostApiRepresentation *jackApi )
         jack_ports = jack_get_ports( jackApi->jack_client, regex_pattern,
                                      NULL, JackPortIsOutput);
         curDevInfo->maxInputChannels = 0;
-        for( i = 0; jack_ports[i] != NULL ; i++)
-        {
-            /* The number of ports returned is the number of output channels.
-             * We don't care what they are, we just care how many */
-            curDevInfo->maxInputChannels++;
+        if (jack_ports) {
+           for( i = 0; jack_ports[i] != NULL ; i++)
+           {
+              /* The number of ports returned is the number of output channels.
+               * We don't care what they are, we just care how many */
+              curDevInfo->maxInputChannels++;
+           }
+           free(jack_ports);
         }
-        free(jack_ports);
 
         /* ... what are your input ports (that we could output to)? */
         jack_ports = jack_get_ports( jackApi->jack_client, regex_pattern,
                                      NULL, JackPortIsInput);
-        curDevInfo->maxOutputChannels = 0;
-        for( i = 0; jack_ports[i] != NULL ; i++)
-        {
-            /* The number of ports returned is the number of input channels.
-             * We don't care what they are, we just care how many */
-            curDevInfo->maxOutputChannels++;
+        if (jack_ports) {
+           curDevInfo->maxOutputChannels = 0;
+           for( i = 0; jack_ports[i] != NULL ; i++)
+           {
+              /* The number of ports returned is the number of input channels.
+               * We don't care what they are, we just care how many */
+              curDevInfo->maxOutputChannels++;
+           }
+           free(jack_ports);
         }
-        free(jack_ports);
 
         curDevInfo->defaultLowInputLatency = 0.;  /* IMPLEMENT ME */
         curDevInfo->defaultLowOutputLatency = 0.;  /* IMPLEMENT ME */
@@ -763,11 +767,17 @@ error:
 static int JackCallback( jack_nframes_t frames, void *userData )
 {
     PaJackStream *stream = (PaJackStream*)userData;
-    PaStreamCallbackTimeInfo timeInfo = {0,0,0}; /* IMPLEMENT ME */
+    PaStreamCallbackTimeInfo timeInfo;
     int callbackResult;
     int chn;
     int framesProcessed;
 
+    /* TODO: make this a lot more accurate */
+    PaTime now = GetStreamTime(stream);
+    timeInfo.currentTime = now;
+    timeInfo.outputBufferDacTime = now;
+    timeInfo.inputBufferAdcTime = now;
+
     if( stream->t0 == -1 )
     {
         if( stream->num_outgoing_connections == 0 )
@@ -904,6 +914,7 @@ static PaError StartStream( PaStream *s )
     }
 
     stream->is_running = TRUE;
+    stream->is_active = TRUE;
 
     return result;
 }
@@ -968,7 +979,8 @@ static PaTime GetStreamTime( PaStream *s )
     PaJackStream *stream = (PaJackStream*)s;
 
     /* TODO: what if we're recording-only? */
-    return jack_frame_time( stream->jack_client ) - stream->t0;
+    int delta_t = jack_frame_time( stream->jack_client ) - stream->t0;
+    return delta_t / (PaTime)jack_get_sample_rate( stream->jack_client );
 }
 
 
