@@ -87,6 +87,7 @@ wxWindow *gParentWindow = NULL;
 ToolBarStub *gControlToolBarStub = NULL;
 ToolBarStub *gMixerToolBarStub = NULL;
 ToolBarStub *gEditToolBarStub = NULL;
+ToolBarStub *gMeterToolBarStub = NULL;
 
 bool gIsQuitting = false;
 
@@ -95,11 +96,13 @@ void SaveWindowSize()
    if(!gAudacityProjects.IsEmpty())
    {
       // BG: Save size of Window 0.
-      wxSize wndSize = gAudacityProjects[0]->GetSize();
-      bool wndMaximized = gAudacityProjects[0]->IsMaximized();
-      gPrefs->Write("/Window/Width", wndSize.GetWidth());
-      gPrefs->Write("/Window/Height", wndSize.GetHeight());
-      gPrefs->Write("/Window/Maximized", wndMaximized);   
+      if (!gAudacityProjects[0]->IsIconized()) {
+         wxSize wndSize = gAudacityProjects[0]->GetSize();
+         bool wndMaximized = gAudacityProjects[0]->IsMaximized();
+         gPrefs->Write("/Window/Width", wndSize.GetWidth());
+         gPrefs->Write("/Window/Height", wndSize.GetHeight());
+         gPrefs->Write("/Window/Maximized", wndMaximized);   
+      }
    }
 }
 
@@ -167,6 +170,11 @@ void QuitAudacity(bool bForce)
       gEditToolBarStub = NULL;
    }
 
+   if (gMeterToolBarStub) {
+      delete gMeterToolBarStub;
+      gMeterToolBarStub = NULL;
+   }
+
    //Delete the clipboard
    AudacityProject::DeleteClipboard();
 
@@ -258,11 +266,15 @@ typedef int (AudacityApp::*SPECIALKEYEVENT)(wxKeyEvent&);
 
 #define globalNewID 4750
 #define globalOpenID 4751
+#define globalPrefsID 4752
+#define globalAboutID 4753
 
 BEGIN_EVENT_TABLE(AudacityApp, wxApp)
 #ifdef __WXMAC__
    EVT_MENU(globalNewID, AudacityApp::OnMenuNew)
    EVT_MENU(globalOpenID, AudacityApp::OnMenuOpen)
+   EVT_MENU(globalPrefsID, AudacityApp::OnMenuPreferences)
+   EVT_MENU(globalAboutID, AudacityApp::OnMenuAbout)
 #endif
 
 END_EVENT_TABLE()
@@ -416,14 +428,14 @@ bool AudacityApp::OnInit()
    LoadEffects();
 
 #ifdef __WXMAC__
-
 #ifdef __MACOSX__
+ #if ((wxMAJOR_VERSION == 2) && (wxMINOR_VERSION <= 4))   
    EnableMenuCommand(NULL, kHICommandPreferences);
-#endif
-
    AEInstallEventHandler(kCoreEventClass,
                          kAEShowPreferences,
-                         NewAEEventHandlerUPP(AEOpenPrefs), 0, 0);   
+                         NewAEEventHandlerUPP(AEOpenPrefs), 0, 0);
+ #endif
+#endif
 
    // Install AppleEvent handlers (allows us to open documents
    // that are dragged to our application's icon)
@@ -435,7 +447,6 @@ bool AudacityApp::OnInit()
                          kAEQuitApplication,
                          NewAEEventHandlerUPP(AEQuit), 0, 0);
 
-
    // On the Mac, users don't expect a program to quit when you close the last window.
    // Create an offscreen frame with a menu bar.  The frame should never
    // be visible, but when all other windows are closed, this menu bar should
@@ -475,6 +486,14 @@ bool AudacityApp::OnInit()
    else
       gMixerToolBarStub = NULL;
 
+   //Only load the meter toolbar if it says so in the preferences
+   bool meterToolBar;
+   gPrefs->Read("/GUI/EnableMeterToolBar", &meterToolBar, true);
+   if(meterToolBar)
+      gMeterToolBarStub =  new ToolBarStub(gParentWindow, MeterToolBarID);
+   else
+      gMeterToolBarStub = NULL;
+
    // Changing the following to NULL will make the application
    // load without the toolbar in memory at all.
 
