@@ -45,6 +45,8 @@
 #include "Project.h"
 #include "Track.h"
 
+#include "MeterToolBar.h"
+
 #include "../AColor.h"
 #include "../images/ControlButtons.h"
 
@@ -639,8 +641,10 @@ void ControlToolBar::SetRecord(bool down)
 void ControlToolBar::PlayPlayRegion(double t0, double t1,
                                     bool looped /* = false */)
 {
-   if (gAudioIO->IsBusy())
+   if (gAudioIO->IsBusy()) {
+      mPlay->PopUp();
       return;
+   }
    
    mStop->Enable();
    mRewind->Disable();
@@ -709,10 +713,11 @@ void ControlToolBar::PlayPlayRegion(double t0, double t1,
             success = true;
             p->SetAudioIOToken(token);
             mBusyProject = p;
-         } else
-         {
+            SetVUMeters(p);
+         }
+         else {
             // msmeyer: Show error message if stream could not be opened
-            wxMessageBox(_("Error while opening sound device. Please check the input "
+            wxMessageBox(_("Error while opening sound device. Please check the output "
                            "device settings and the project sample rate."),
                          _("Error"), wxOK | wxICON_EXCLAMATION, this);
          }
@@ -748,7 +753,16 @@ void ControlToolBar::OnPlay(wxCommandEvent &evt)
       PlayCurrentRegion(false);
 }
 
-
+void ControlToolBar::SetVUMeters(AudacityProject *p)
+{
+   MeterToolBar *bar;
+   bar = p->GetMeterToolBar();
+   if (bar) {
+      Meter *play, *record;
+      bar->GetMeters(&play, &record);
+      gAudioIO->SetMeters(record, play);
+   }
+}
 
 void ControlToolBar::PlayCurrentRegion(bool looped /* = false */)
 {
@@ -789,8 +803,10 @@ void ControlToolBar::StopPlaying()
 
 void ControlToolBar::OnRecord(wxCommandEvent &evt)
 {
-   if (gAudioIO->IsBusy())
+   if (gAudioIO->IsBusy()) {
+      mRecord->PopUp();
       return;
+   }
 
    mPlay->Disable();
    mStop->Enable();
@@ -858,10 +874,11 @@ void ControlToolBar::OnRecord(wxCommandEvent &evt)
       if (success) {
          p->SetAudioIOToken(token);
          mBusyProject = p;
+         SetVUMeters(p);
       }
       else {
          // msmeyer: Show error message if stream could not be opened
-         wxMessageBox(_("Error while opening sound device. Please check the output "
+         wxMessageBox(_("Error while opening sound device. Please check the input "
                         "device settings and the project sample rate."),
                       _("Error"), wxOK | wxICON_EXCLAMATION, this);
 
@@ -896,9 +913,6 @@ void ControlToolBar::OnRewind(wxCommandEvent &evt)
 
    AudacityProject *p = GetActiveProject();
    if (p) {
-      if (gAudioIO->IsStreamActive(p->GetAudioIOToken()))
-         StopPlaying();
-      
       p->Rewind(mRewind->WasShiftDown());
    }
 }
@@ -911,9 +925,6 @@ void ControlToolBar::OnFF(wxCommandEvent &evt)
    AudacityProject *p = GetActiveProject();
 
    if (p) {
-      if (gAudioIO->IsStreamActive(p->GetAudioIOToken()))
-         StopPlaying();
-      
       p->SkipEnd(mFF->WasShiftDown());
    }
 }
