@@ -23,6 +23,7 @@
 #include <wx/filedlg.h>
 #include <wx/textfile.h>
 #include <wx/progdlg.h>
+#include <wx/scrolbar.h>
 
 #include "Project.h"
 
@@ -31,6 +32,7 @@
 #include "import/ImportMIDI.h"
 #include "import/ImportRaw.h"
 #include "export/Export.h"
+#include "export/ExportMultiple.h"
 #include "prefs/PrefsDialog.h"
 #include "HistoryWindow.h"
 #include "Internat.h"
@@ -45,6 +47,12 @@
 #include "Help.h"
 #include "Benchmark.h"
 
+// Printing
+#include <wx/print.h>
+#include <wx/printdlg.h>
+#include "TrackArtist.h"
+#include "widgets/Ruler.h"
+
 #include "Resample.h"
 
 enum {
@@ -107,7 +115,9 @@ enum {
    UnsavedChangesFlag     = 0x00000080,
    HasLastEffectFlag      = 0x00000100,
    UndoAvailableFlag      = 0x00000200,
-   RedoAvailableFlag      = 0x00000400
+   RedoAvailableFlag      = 0x00000400,
+   ZoomInAvailableFlag    = 0x00000800,
+   ZoomOutAvailableFlag   = 0x00001000
 };
 
 #define FN(X) new AudacityProjectCommandFunctor(this, &AudacityProject:: X )
@@ -151,7 +161,7 @@ void AudacityProject::CreateMenusAndCommands()
    c->AddItem("ExportOggSel",   _("Export Selection As Ogg Vorbis..."), FN(OnExportOggSelection));
    c->AddSeparator();
    c->AddItem("ExportLabels",   _("Export &Labels..."),              FN(OnExportLabels));
-
+   c->AddItem("ExportMultiple",   _("Export &Multiple..."),              FN(OnExportMultiple));
    // Enable Export commands only when there are tracks
    c->SetCommandFlags(AudioIONotBusyFlag | TracksExistFlag,
                       AudioIONotBusyFlag | TracksExistFlag,
@@ -164,10 +174,35 @@ void AudacityProject::CreateMenusAndCommands()
    c->SetCommandFlags("ExportLabels",
                       AudioIONotBusyFlag | LabelTracksExistFlag,
                       AudioIONotBusyFlag | LabelTracksExistFlag);
+   c->SetCommandFlags("ExportMultiple",
+                      AudioIONotBusyFlag | TracksExistFlag,
+                      AudioIONotBusyFlag | TracksExistFlag);
+
+   c->AddSeparator();
+   c->AddItem("PageSetup",   _("Page Setup..."),              FN(OnPageSetup));
+   c->AddItem("Print",       _("Print..."),                   FN(OnPrint));
+   c->SetCommandFlags("PageSetup",
+                      AudioIONotBusyFlag | TracksExistFlag,
+                      AudioIONotBusyFlag | TracksExistFlag);
+   c->SetCommandFlags("Print",
+                      AudioIONotBusyFlag | TracksExistFlag,
+                      AudioIONotBusyFlag | TracksExistFlag);
 
    // On Mac OS X, Preferences and Quit are in the application menu,
-   // not the File menu.
-  #ifndef __WXMAC__
+   // not the File menu.  In wx 2.4 and lower, we handle this by
+   // not including these menus at all.  In wx 2.5 and higher, we
+   // include them, but wxMac automatically moves them to the appropriate
+   // place.
+
+#ifdef __WXMAC__
+ #if ((wxMAJOR_VERSION == 2) && (wxMINOR_VERSION <= 4))
+   // Leave these menus out entirely...
+ #else
+   c->AddItem("Preferences",    _("&Preferences...\tCtrl+P"),        FN(OnPreferences));
+   c->AddItem("Exit",           _("E&xit"),                          FN(OnExit));
+   c->SetCommandFlags("Exit", 0, 0);
+ #endif
+#else
    c->AddSeparator();
    c->AddItem("Preferences",    _("&Preferences...\tCtrl+P"),        FN(OnPreferences));
    c->AddSeparator();
@@ -278,8 +313,12 @@ void AudacityProject::CreateMenusAndCommands()
    c->BeginMenu(_("&View"));
    c->SetDefaultFlags(0, 0);
    c->AddItem("ZoomIn",         _("Zoom &In\tCtrl+1"),               FN(OnZoomIn));
+   c->SetCommandFlags("ZoomIn", ZoomInAvailableFlag, ZoomInAvailableFlag);
+
    c->AddItem("ZoomNormal",     _("Zoom &Normal\tCtrl+2"),           FN(OnZoomNormal));
    c->AddItem("ZoomOut",        _("Zoom &Out\tCtrl+3"),              FN(OnZoomOut));
+   c->SetCommandFlags("ZoomOut", ZoomOutAvailableFlag, ZoomOutAvailableFlag);
+
    c->AddItem("FitInWindow",    _("&Fit in Window\tCtrl+F"),         FN(OnZoomFit));
    c->AddItem("FitV",           _("Fit &Vertically\tCtrl+Shift+F"),  FN(OnZoomFitV));
    c->AddItem("ZoomSel",        _("&Zoom to Selection\tCtrl+E"),     FN(OnZoomSel));
@@ -299,6 +338,10 @@ void AudacityProject::CreateMenusAndCommands()
    c->AddItem("FloatControlTB", _("Float Control Toolbar"),          FN(OnFloatControlToolBar));
    c->AddItem("FloatEditTB",    _("Float Edit Toolbar"),             FN(OnFloatEditToolBar));
    c->AddItem("FloatMixerTB",   _("Float Mixer Toolbar"),            FN(OnFloatMixerToolBar));
+   c->AddItem("FloatMeterTB",   _("Float Meter Toolbar"),            FN(OnFloatMeterToolBar));
+
+   c->SetCommandFlags("FloatMeterTB", AudioIONotBusyFlag, AudioIONotBusyFlag);
+
    c->EndMenu();
 
    //
@@ -358,7 +401,9 @@ void AudacityProject::CreateMenusAndCommands()
 
    c->AddSeparator();   
    c->AddItem("AddLabel",       _("Add Label At Selection\tCtrl+B"), FN(OnAddLabel));
+   c->AddItem("AddLabelPlaying",       _("Add Label At Playback Position\tCtrl+M"), FN(OnAddLabelPlaying));
    c->SetCommandFlags("AddLabel", 0, 0);
+   c->SetCommandFlags("AddLabelPlaying", 0, AudioIONotBusyFlag);
    c->EndMenu();
 
    //
@@ -440,11 +485,12 @@ void AudacityProject::CreateMenusAndCommands()
    delete effects;
    c->EndMenu();
 
+   /* i18n-hint: The name of the Help menu */
    c->BeginMenu(_("&Help"));
    c->SetDefaultFlags(0, 0);
-   c->AddItem("About",          _("&About Audacity..."),          FN(OnAbout));
-   c->AddSeparator();   
    c->AddItem("Help",           _("&Online Help..."),             FN(OnHelp));
+   c->AddSeparator();   
+   c->AddItem("About",          _("&About Audacity..."),          FN(OnAbout));
 
 #if 0 // No Benchmark in stable release
    c->AddSeparator();   
@@ -453,6 +499,12 @@ void AudacityProject::CreateMenusAndCommands()
 
    c->EndMenu();
 
+#ifdef __WXMAC__
+ #if (!((wxMAJOR_VERSION == 2) && (wxMINOR_VERSION <= 4)))
+   wxGetApp().s_macHelpMenuTitleName = _("&Help");
+ #endif
+#endif
+
    ModifyExportMenus();
 
    SetMenuBar(menubar);
@@ -606,6 +658,12 @@ wxUint32 AudacityProject::GetUpdateFlags()
    if (mUndoManager.RedoAvailable())
       flags |= RedoAvailableFlag;
 
+   if (GetZoom() < gMaxZoom && (flags & TracksExistFlag))
+      flags |= ZoomInAvailableFlag;
+
+   if (GetZoom() > gMinZoom && (flags & TracksExistFlag))
+      flags |= ZoomOutAvailableFlag;
+
    return flags;
 }
 
@@ -613,21 +671,24 @@ int AudacityProject::GetToolBarChecksum()
 {
    //Calculate the ToolBarCheckSum (uniquely specifies state of all toolbars):
    int toolBarCheckSum = 0;
-   toolBarCheckSum += gControlToolBarStub->GetWindowedStatus() ? 2 : 1;
+   if (gControlToolBarStub->GetWindowedStatus())
+      toolBarCheckSum += 1;
    if (gEditToolBarStub) {
       if (gEditToolBarStub->GetLoadedStatus()) {
          if(gEditToolBarStub->GetWindowedStatus())
-            toolBarCheckSum += 6;
-         else
-            toolBarCheckSum += 3;
+            toolBarCheckSum += 2;
       }
    }
    if (gMixerToolBarStub) {
       if (gMixerToolBarStub->GetLoadedStatus()) {
          if(gMixerToolBarStub->GetWindowedStatus())
-            toolBarCheckSum += 12;
-         else
-            toolBarCheckSum += 24;
+            toolBarCheckSum += 4;
+      }
+   }
+   if (gMeterToolBarStub) {
+      if (gMeterToolBarStub->GetLoadedStatus()) {
+         if(gMeterToolBarStub->GetWindowedStatus())
+            toolBarCheckSum += 8;
       }
    }
 
@@ -665,10 +726,28 @@ void AudacityProject::ModifyToolbarMenus()
    else {
       mCommandManager.Enable("FloatMixerTB", false);
    }   
+
+   if (gMeterToolBarStub) {
+     
+     // Loaded or unloaded?
+     mCommandManager.Enable("FloatMeterTB", gMeterToolBarStub->GetLoadedStatus());
+     
+     // Floating or docked?
+     if (gMeterToolBarStub->GetWindowedStatus())
+        mCommandManager.Modify("FloatMeterTB", _("Dock Meter Toolbar"));
+     else
+        mCommandManager.Modify("FloatMeterTB", _("Float Meter Toolbar"));
+   }
+   else {
+      mCommandManager.Enable("FloatMeterTB", false);
+   }   
 }
 
 void AudacityProject::UpdateMenus()
 {
+   if (this != GetActiveProject())
+      return;
+
    if (gControlToolBarStub) {
       int toolBarCheckSum = GetToolBarChecksum();
 
@@ -1329,12 +1408,188 @@ void AudacityProject::OnExportOggSelection()
    ::ExportLossy(this, true, mViewInfo.sel0, mViewInfo.sel1);
 }
 
+void AudacityProject::OnExportMultiple()
+{
+   ::ExportMultiple(this);
+}
+
 void AudacityProject::OnPreferences()
 {
    PrefsDialog dialog(this /* parent */ );
    dialog.ShowModal();
 }
 
+//
+// Printing
+//
+
+// Globals, so that we remember settings from session to session
+wxPrintData *gPrintData = NULL;
+wxPageSetupData *gPageSetupData = NULL;
+
+class AudacityPrintout : public wxPrintout
+{
+ public:
+   AudacityPrintout(wxString title,
+                    TrackList *tracks):
+      wxPrintout(title),
+      mTracks(tracks)
+   {
+   }
+   bool OnPrintPage(int page);
+   bool HasPage(int page);
+   bool OnBeginDocument(int startPage, int endPage);
+   void GetPageInfo(int *minPage, int *maxPage,
+                    int *selPageFrom, int *selPageTo);
+
+ private:
+   TrackList *mTracks;
+};
+
+bool AudacityPrintout::OnPrintPage(int page)
+{
+   wxDC *dc = GetDC();
+   if (!dc)
+      return false;
+
+   int width, height;
+   dc->GetSize(&width, &height);
+
+   int rulerScreenHeight = 40;
+   int screenTotalHeight = mTracks->GetHeight() + rulerScreenHeight;
+
+   double scale = height / (double)screenTotalHeight;
+
+   int rulerPageHeight = (int)(rulerScreenHeight * scale);
+   Ruler ruler;
+   ruler.SetBounds(0, 0, width, rulerPageHeight);
+   ruler.SetOrientation(wxHORIZONTAL);
+   ruler.SetRange(0.0, mTracks->GetEndTime());
+   ruler.SetFormat(Ruler::TimeFormat);
+   ruler.SetLabelEdges(true);
+   ruler.Draw(*dc);
+
+   TrackArtist artist;
+   artist.SetBackgroundBrushes(*wxWHITE_BRUSH, *wxWHITE_BRUSH,
+			       *wxWHITE_PEN, *wxWHITE_PEN);
+   ViewInfo viewInfo;
+   viewInfo.sel0 = viewInfo.sel1 = 0;
+   viewInfo.vpos = 0;
+   viewInfo.h = 0.0;
+   viewInfo.screen = mTracks->GetEndTime() - viewInfo.h;
+   viewInfo.total = viewInfo.screen;
+   viewInfo.zoom = viewInfo.lastZoom = width / viewInfo.screen;
+   int y = rulerPageHeight;
+
+   TrackListIterator iter(mTracks);
+   Track *n = iter.First();
+   while (n) {
+      wxRect r;
+      r.x = 0;
+      r.y = y;
+      r.width = width;
+      r.height = (int)(n->GetHeight() * scale);
+
+      switch(n->GetKind()) {
+      case Track::Wave:
+         switch (((WaveTrack *)n)->GetDisplay()) {
+         case WaveTrack::WaveformDisplay:
+            artist.DrawWaveform((WaveTrack *)n, *dc, r,
+                                &viewInfo, false, false, false, false, false);
+            break;
+         case WaveTrack::WaveformDBDisplay:
+            artist.DrawWaveform((WaveTrack *)n, *dc, r,
+                                &viewInfo, false, false, false, true, false);
+            break;
+         case WaveTrack::SpectrumDisplay:
+            artist.DrawSpectrum((WaveTrack *)n, *dc, r, &viewInfo, false);
+            break;
+         case WaveTrack::PitchDisplay:
+            artist.DrawSpectrum((WaveTrack *)n, *dc, r, &viewInfo, true);
+            break;
+         }
+         break;
+      case Track::Note:
+         artist.DrawNoteTrack((NoteTrack *)n, *dc, r, &viewInfo);
+         break;
+      case Track::Label:
+         artist.DrawLabelTrack((LabelTrack *)n, *dc, r, &viewInfo);
+         break;
+      case Track::Time:
+         artist.DrawTimeTrack((TimeTrack *)n, *dc, r, &viewInfo);
+         break;
+      }
+
+      dc->SetPen(*wxBLACK_PEN);
+      dc->DrawLine(0, r.y, width, r.y);
+
+      n = iter.Next();
+      y += r.height;
+   };
+
+   return true;
+}
+
+bool AudacityPrintout::HasPage(int page)
+{
+   return (page==1);
+}
+
+bool AudacityPrintout::OnBeginDocument(int startPage, int endPage)
+{
+   return wxPrintout::OnBeginDocument(startPage, endPage);
+}
+
+void AudacityPrintout::GetPageInfo(int *minPage, int *maxPage,
+                                   int *selPageFrom, int *selPageTo)
+{
+   *minPage = 1;
+   *maxPage = 1;
+   *selPageFrom = 1;
+   *selPageTo = 1;
+}
+
+void AudacityProject::OnPageSetup()
+{
+   if (gPageSetupData == NULL)
+      gPageSetupData = new wxPageSetupDialogData();
+   if (gPrintData == NULL)
+      gPrintData = new wxPrintData();
+
+   (*gPageSetupData) = *gPrintData;
+
+   wxPageSetupDialog pageSetupDialog(this, gPageSetupData);
+   pageSetupDialog.ShowModal();
+
+   (*gPrintData) = pageSetupDialog.GetPageSetupData().GetPrintData();
+   (*gPageSetupData) = pageSetupDialog.GetPageSetupData();
+}
+
+void AudacityProject::OnPrint()
+{
+   if (gPageSetupData == NULL)
+      gPageSetupData = new wxPageSetupDialogData();
+   if (gPrintData == NULL)
+      gPrintData = new wxPrintData();
+
+   wxPrintDialogData printDialogData(*gPrintData);
+
+   wxPrinter printer(&printDialogData);
+   AudacityPrintout printout(GetName(), mTracks);
+   if (!printer.Print(this, &printout, true)) {
+      if (wxPrinter::GetLastError() == wxPRINTER_ERROR) {
+         wxMessageBox(_("There was a problem printing."),
+                      _("Print"), wxOK);
+      }
+      else {
+         // Do nothing, the user cancelled...
+      }
+   }
+   else {
+      *gPrintData = printer.GetPrintDialogData().GetPrintData();
+   }
+}
+
 //
 // Edit Menu
 //
@@ -1395,10 +1650,11 @@ void AudacityProject::OnCut()
    TrackListIterator iter(mTracks);
 
    Track *n = iter.First();
-   Track *dest = 0;
+   Track *dest;
 
    while (n) {
       if (n->GetSelected()) {
+         dest = NULL;
          n->Cut(mViewInfo.sel0, mViewInfo.sel1, &dest);
          if (dest) {
             dest->SetChannel(n->GetChannel());
@@ -1433,10 +1689,11 @@ void AudacityProject::OnCopy()
    TrackListIterator iter(mTracks);
 
    Track *n = iter.First();
-   Track *dest = 0;
+   Track *dest;
 
    while (n) {
       if (n->GetSelected()) {
+         dest = NULL;
          n->Copy(mViewInfo.sel0, mViewInfo.sel1, &dest);
          if (dest) {
             dest->SetChannel(n->GetChannel());
@@ -1642,12 +1899,13 @@ void AudacityProject::OnDuplicate()
    TrackListIterator iter(mTracks);
 
    Track *n = iter.First();
-   Track *dest = 0;
+   Track *dest;
 
    TrackList newTracks;
 
    while (n) {
       if (n->GetSelected()) {
+         dest = NULL;
          n->Copy(mViewInfo.sel0, mViewInfo.sel1, &dest);
          if (dest) {
             dest->Init(*n);
@@ -1676,7 +1934,7 @@ void AudacityProject::OnSplit()
    TrackListIterator iter(mTracks);
 
    Track *n = iter.First();
-   Track *dest = 0;
+   Track *dest;
 
    TrackList newTracks;
 
@@ -1685,6 +1943,7 @@ void AudacityProject::OnSplit()
          double sel0 = mViewInfo.sel0;
          double sel1 = mViewInfo.sel1;
 
+         dest = NULL;
          n->Copy(sel0, sel1, &dest);
          if (dest) {
             dest->Init(*n);
@@ -2010,6 +2269,7 @@ void AudacityProject::OnZoomFitV()
       t = iter2.Next();
    }
 
+   mVsbar->SetThumbPosition(0);
    FixScrollbars();
    Refresh(false);
    ModifyState();
@@ -2228,6 +2488,62 @@ void AudacityProject::OnFloatMixerToolBar()
    }
 }
 
+void AudacityProject::OnLoadMeterToolBar()
+{
+   if (gMeterToolBarStub) {
+      if (gMeterToolBarStub->GetLoadedStatus()) {
+
+         //the toolbar is "loaded", meaning its visible either in the window
+         //or floating
+
+         gMeterToolBarStub->SetLoadedStatus(false);
+         gMeterToolBarStub->HideWindowedToolBar();
+         gMeterToolBarStub->UnloadAll();
+
+      } else {
+
+         //the toolbar is not "loaded", meaning that although the stub exists, 
+         //the toolbar is not visible either in a window or floating around
+         gMeterToolBarStub->SetLoadedStatus(true);
+
+         if (gMeterToolBarStub->GetWindowedStatus()) {
+            //Make the floating toolbar appear
+            gMeterToolBarStub->ShowWindowedToolBar();
+            gMeterToolBarStub->LoadAll();
+         } else {
+            //Make it appear in all the windows
+            gMeterToolBarStub->LoadAll();
+         }
+
+      }
+   } else {
+      gMeterToolBarStub = new ToolBarStub(gParentWindow, MeterToolBarID);
+      gMeterToolBarStub->LoadAll();
+   }
+}
+
+
+void AudacityProject::OnFloatMeterToolBar()
+{
+   // Can't drag the Meter toolbar while Audio I/O is busy at all
+   if (gAudioIO->IsStreamActive())
+      return;
+
+   if (gMeterToolBarStub) {
+
+      if (gMeterToolBarStub->GetWindowedStatus()) {
+
+         gMeterToolBarStub->HideWindowedToolBar();
+         gMeterToolBarStub->LoadAll();
+
+      } else {
+
+         gMeterToolBarStub->ShowWindowedToolBar();
+         gMeterToolBarStub->UnloadAll();
+      }
+   }
+}
+
 
 
 //
@@ -2396,7 +2712,8 @@ void AudacityProject::OnImportRaw()
 
 void AudacityProject::OnEditID3()
 {
-   mTags->ShowEditDialog(this, _("Edit ID3 Tags (for MP3 exporting)"));
+   if (mTags->ShowEditDialog(this, _("Edit ID3 Tags (for MP3 exporting)")))
+      PushState(_("Edit ID3 tags"), _("Edit ID3 Tags"));
 }
 
 void AudacityProject::OnQuickMix()
@@ -2709,7 +3026,7 @@ void AudacityProject::OnNewTimeTrack()
       }
 }
 
-void AudacityProject::OnAddLabel()
+void AudacityProject::DoAddLabel(double left, double right)
 {
    TrackListIterator iter(mTracks);
    LabelTrack *lt = NULL;
@@ -2730,7 +3047,7 @@ void AudacityProject::OnAddLabel()
    SelectNone();
    lt->SetSelected(true);
 
-   lt->AddLabel(mViewInfo.sel0, mViewInfo.sel1);
+   lt->AddLabel(left, right);
 
    PushState(_("Added label"), _("Label"));
 
@@ -2738,6 +3055,20 @@ void AudacityProject::OnAddLabel()
    mTrackPanel->Refresh(false);
 }
 
+void AudacityProject::OnAddLabel()
+{
+   DoAddLabel(mViewInfo.sel0, mViewInfo.sel1);
+}
+
+void AudacityProject::OnAddLabelPlaying()
+{
+   if (GetAudioIOToken()>0 &&
+       gAudioIO->IsStreamActive(GetAudioIOToken())) {
+      double indicator = gAudioIO->GetStreamTime();
+      DoAddLabel(indicator, indicator);
+   }
+}
+
 void AudacityProject::OnRemoveTracks()
 {
    TrackListIterator iter(mTracks);
