@@ -63,6 +63,7 @@
 #include "AudioIO.h"
 #include "ControlToolBar.h"
 #include "EditToolBar.h"
+#include "MeterToolBar.h"
 #include "FormatSelection.h"
 #include "FreqWindow.h"
 #include "HistoryWindow.h"
@@ -86,7 +87,6 @@
 #include "PlatformCompatibility.h"
 #include <wx/arrimpl.cpp>       // this allows for creation of wxObjArray
 
-
 TrackList *AudacityProject::msClipboard = new TrackList();
 double AudacityProject::msClipLen = 0.0;
 AudacityProject *AudacityProject::msClipProject = NULL;
@@ -409,6 +409,20 @@ AudacityProject::AudacityProject(wxWindow * parent, wxWindowID id,
 //      mTotalToolBarHeight += h;
    }
 
+   if (gMeterToolBarStub) {
+      if (gMeterToolBarStub->GetLoadedStatus()
+          && !gMeterToolBarStub->GetWindowedStatus()) {
+         int h = gMeterToolBarStub->GetHeight();
+         ToolBar *etb = new MeterToolBar(this, 0, wxPoint(10, top),
+                                        wxSize(width - 10 - sbarSpaceWidth, h));
+         mToolBarArray.Add((ToolBar *) etb);
+
+//         top += h + 1;
+//         height -= h + 1;
+//         mTotalToolBarHeight += h;
+      }
+   }
+
    if (gMixerToolBarStub) {
       if (gMixerToolBarStub->GetLoadedStatus()
           && !gMixerToolBarStub->GetWindowedStatus()) {
@@ -970,6 +984,9 @@ void AudacityProject::OnIconize(wxIconizeEvent &event)
       if (gEditToolBarStub) {
          gEditToolBarStub->Iconize( mIconized );
       }
+      if (gMeterToolBarStub) {
+         gMeterToolBarStub->Iconize( mIconized );
+      }
    }
 
    event.Skip();
@@ -1021,8 +1038,11 @@ void AudacityProject::OnScroll(wxScrollEvent & event)
 
       mTrackPanel->Refresh(false);
 #ifdef __WXMAC__
+ #if ((wxMAJOR_VERSION == 2) && (wxMINOR_VERSION <= 4))
       mTrackPanel->MacUpdateImmediately();
+ #endif
 #endif
+
    }
 }
 
@@ -1290,7 +1310,7 @@ int AudacityProject::FlowLayout( int i, int x, int y, int width, int height )
                mToolBarArray[i-2]->GetPosition( &bar2X, &bar2Y );
                if( bar2Y == barY ){
                   int dummy;
-                  mToolBarArray[i-2]->GetSize( &barWidth, &dummy );
+                  mToolBarArray[i-2]->GetIdealSize( &barWidth, &dummy );
                   mToolBarArray[i-2]->SetSize( barWidth,  barHeight + height+extraSpace);
 //                  wxLogDebug("b: At %i,%i for %i Toolbar %i has adjusted height %i", x,y,bar2Y, i-2, barHeight + height + extraSpace );
                }
@@ -1420,6 +1440,22 @@ void AudacityProject::LoadToolBar(enum ToolBarType t)
      break;
       
      
+   case MeterToolBarID:
+
+     if (!gMeterToolBarStub) {
+       gMeterToolBarStub = new ToolBarStub(gParentWindow, MeterToolBarID);
+     }
+     
+     h = gMeterToolBarStub->GetHeight();
+     toolbar =
+       new MeterToolBar(this, -1, wxPoint(10, tbheight),
+			wxSize(width - 10, h));
+     
+     
+     mToolBarArray.Insert(toolbar, 1);
+     break;
+      
+     
    case NoneID:
    default:
      toolbar = NULL;
@@ -1463,6 +1499,8 @@ void AudacityProject::UnloadToolBar(enum ToolBarType t)
          case MixerToolBarID:
             break;
 
+         case MeterToolBarID:
+            break;
 
          case NoneID:
          default:
@@ -1498,8 +1536,15 @@ int AudacityProject::GetGrabberFromEvent(wxMouseEvent & event)
    {
       wxRect r = mToolBarArray[i]->GetRect();
       if(( r.y < event.m_y )  && ( event.m_y < (r.y+r.height) ) &&
-         ( (r.x- grabberWidth) < event.m_x) && (event.m_x < r.x ))
-         return i;
+         ( (r.x- grabberWidth) < event.m_x) && (event.m_x < r.x )) {
+
+         // Can't drag the Meter toolbar while Audio I/O is busy at all,
+         // so don't return it in this case...
+         if (gAudioIO->IsStreamActive())
+            return -1;
+         else
+            return i;
+      }
    }
    return -1;
 }
@@ -1591,6 +1636,10 @@ void AudacityProject::OnMouseEvent(wxMouseEvent & event)
          gMixerToolBarStub->ShowWindowedToolBar(&mouse);
          gMixerToolBarStub->UnloadAll();
          break;
+      case MeterToolBarID:
+         gMeterToolBarStub->ShowWindowedToolBar(&mouse);
+         gMeterToolBarStub->UnloadAll();
+         break;
 
       case NoneID:
       default:
@@ -2536,6 +2585,18 @@ MixerToolBar *AudacityProject::GetMixerToolBar()
    return NULL;
 }
 
+MeterToolBar *AudacityProject::GetMeterToolBar()
+{
+   for(unsigned int i=0; i<mToolBarArray.GetCount(); i++)
+      if ((mToolBarArray[i]->GetType()) == MeterToolBarID)
+         return (MeterToolBar *)mToolBarArray[i];
+
+   if (gMeterToolBarStub)
+      return (MeterToolBar *)gMeterToolBarStub->GetToolBar();
+
+   return NULL;
+}
+
 void AudacityProject::SetStop(bool bStopped)
 {
    mTrackPanel->SetStop(bStopped);
