@@ -1349,8 +1349,11 @@ bool ExportMP3(AudacityProject *project,
    /* Put ID3 tags at beginning of file */
    
    Tags *tags = project->GetTags();
-   if (!tags->ShowEditDialog(project, _("Edit the ID3 tags for the MP3 file")))
-      return false;  // used selected "cancel"
+   if (tags->IsEmpty()) {
+      if (!tags->ShowEditDialog(project,
+                                _("Edit the ID3 tags for the MP3 file")))
+         return false;  // used selected "cancel"
+   }
 
    char *id3buffer = NULL;
    int id3len;
